import argparse
import json
import sys

from pyhydroquebec import HydroQuebecClient, REQUESTS_TIMEOUT


def _format_output(account, data):
    """Format data to get a readable output"""
    data['account'] = account
    output = ("""Hydro Quebec data for account: {d[account]}

Period Info
===========
Period day number:         {d[period_length]:d}
Period total days:     {d[period_total_days]:d} days

Period current bill
===================
Total Bill: {d[period_total_bill]:.2f} $
Mean Daily Bill: {d[period_mean_daily_bill]:.2f} $

Total period consumption
========================
Lower price: {d[period_lower_price_consumption]:.2f} kWh
Higher price: {d[period_higher_price_consumption]:.2f} kWh
Total: {d[period_total_consumption]:.2f} kWh
Mean daily: {d[period_mean_daily_consumption]:.2f} kWh

Yesterday consumption
=====================
Lower price:  {d[yesterday_lower_price_consumption]:.2f} kWh
Higher price: {d[yesterday_higher_price_consumption]:.2f} kWh
Total:  {d[yesterday_total_consumption]:.2f} kWh
""")
    print(output.format(d=data))


def main():
    """Main function"""
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--username',
                        required=True, help='Hydro Quebec username')
    parser.add_argument('-p', '--password',
                        required=True, help='Password')
    parser.add_argument('-j', '--json', action='store_true',
                        default=False, help='Json output')
    parser.add_argument('-t', '--timeout',
                        default=REQUESTS_TIMEOUT, help='Request timeout')
    args = parser.parse_args()
    client = HydroQuebecClient(args.username, args.password, args.timeout)
    client.fetch_data()
    if not client.get_data():
        return
    if args.json:
        print(json.dumps(client.get_data()))
    else:
        _format_output(args.username, client.get_data())


if __name__ == '__main__':
    sys.exit(main())



"""
PyHydroQuebec
"""
import json
import logging
import re

from bs4 import BeautifulSoup
import requests

REQUESTS_TIMEOUT = 15

HOST = "https://www.hydroquebec.com"
HOME_URL = "{}/portail/web/clientele/authentification".format(HOST)
PROFILE_URL = ("{}/portail/fr/group/clientele/"
               "portrait-de-consommation".format(HOST))
MONTHLY_MAP = (('period_total_bill', 'montantFacturePeriode'),
               ('period_length', 'nbJourLecturePeriode'),
               ('period_total_days', 'nbJourPrevuPeriode'),
               ('period_mean_daily_bill', 'moyenneDollarsJourPeriode'),
               ('period_mean_daily_consumption', 'moyenneKwhJourPeriode'),
               ('period_total_consumption', 'consoTotalPeriode'),
               ('period_lower_price_consumption', 'consoRegPeriode'),
               ('period_higher_price_consumption', 'consoHautPeriode'))
DAILY_MAP = (('yesterday_total_consumption', 'consoTotalQuot'),
             ('yesterday_lower_price_consumption', 'consoRegQuot'),
             ('yesterday_higher_price_consumption', 'consoHautQuot'))


class PyHydroQuebecError(Exception):
    pass


class HydroQuebecClient(object):

    def __init__(self, username, password, timeout=REQUESTS_TIMEOUT):
        """Initialize the client object."""
        self.username = username
        self.password = password
        self._data = {}
        self._cookies = None
        self._timeout = timeout

    def _get_login_page(self):
        """Go to the login page."""
        try:
            raw_res = requests.get(HOME_URL, timeout=REQUESTS_TIMEOUT)
        except OSError:
            raise PyHydroQuebecError("Can not connect to login page")
        # Get cookies
        self._cookies = raw_res.cookies
        # Get login url
        soup = BeautifulSoup(raw_res.content, 'html.parser')
        form_node = soup.find('form', {'name': 'fm'})
        if form_node is None:
            raise PyHydroQuebecError("No login form find")
        login_url = form_node.attrs.get('action')
        if login_url is None:
            raise PyHydroQuebecError("Can not found login url")
        return login_url

    def _post_login_page(self, login_url):
        """Login to HydroQuebec website."""
        data = {"login": self.username,
                "_58_password": self.password}

        try:
            raw_res = requests.post(login_url,
                                    data=data,
                                    cookies=self._cookies,
                                    allow_redirects=False,
                                    timeout=REQUESTS_TIMEOUT)
        except OSError:
            raise PyHydroQuebecError("Can not submit login form")
        if raw_res.status_code != 302:
            raise PyHydroQuebecError("Bad HTTP status code")

        # Update cookies
        self._cookies.update(raw_res.cookies)
        return True

    def _get_p_p_id(self):
        """Get id of consumption profile."""
        from bs4 import BeautifulSoup
        try:
            raw_res = requests.get(PROFILE_URL,
                                   cookies=self._cookies,
                                   timeout=REQUESTS_TIMEOUT)
        except OSError:
            raise PyHydroQuebecError("Can not get profile page")
        # Update cookies
        self._cookies.update(raw_res.cookies)
        # Looking for p_p_id
        soup = BeautifulSoup(raw_res.content, 'html.parser')
        p_p_id = None
        for node in soup.find_all('span'):
            node_id = node.attrs.get('id', "")
            if node_id.startswith("p_portraitConsommation_WAR"):
                p_p_id = node_id[2:]
                break

        if p_p_id is None:
            raise PyHydroQuebecError("Could not get p_p_id")

        return p_p_id

    def _get_monthly_data(self, p_p_id):
        """Get monthly data."""
        params = {"p_p_id": p_p_id,
                  "p_p_lifecycle": 2,
                  "p_p_resource_id": ("resourceObtenirDonnees"
                                      "PeriodesConsommation")}
        try:
            raw_res = requests.get(PROFILE_URL,
                                   params=params,
                                   cookies=self._cookies,
                                   timeout=REQUESTS_TIMEOUT)
        except OSError:
            raise PyHydroQuebecError("Can not get monthly data")
        try:
            json_output = raw_res.json()
        except OSError:
            raise PyHydroQuebecError("Could not get monthly data")

        if not json_output.get('success'):
            raise PyHydroQuebecError("Could not get monthly data")

        return json_output.get('results')

    def _get_daily_data(self, p_p_id, start_date, end_date):
        """Get daily data."""
        params = {"p_p_id": p_p_id,
                  "p_p_lifecycle": 2,
                  "p_p_resource_id":
                  "resourceObtenirDonneesQuotidiennesConsommation",
                  "dateDebutPeriode": start_date,
                  "dateFinPeriode": end_date}
        try:
            raw_res = requests.get(PROFILE_URL,
                                   params=params,
                                   cookies=self._cookies,
                                   timeout=REQUESTS_TIMEOUT)
        except OSError:
            raise PyHydroQuebecError("Can not get daily data")
        try:
            json_output = raw_res.json()
        except OSError:
            raise PyHydroQuebecError("Could not get daily data")

        if not json_output.get('success'):
            raise PyHydroQuebecError("Could not get daily data")

        return json_output.get('results')

    def fetch_data(self):
        """Get the latest data from HydroQuebec."""
        # Get login page
        login_url = self._get_login_page()
        # Post login page
        self._post_login_page(login_url)
        # Get p_p_id
        p_p_id = self._get_p_p_id()
        # Get Monthly data
        monthly_data = self._get_monthly_data(p_p_id)[0]
        # Get daily data
        start_date = monthly_data.get('dateDebutPeriode')
        end_date = monthly_data.get('dateFinPeriode')
        daily_data = self._get_daily_data(p_p_id, start_date, end_date)
        daily_data = daily_data[0]['courant']

        # format data
        for key1, key2 in MONTHLY_MAP:
            self._data[key1] = monthly_data[key2]
        for key1, key2 in DAILY_MAP:
            self._data[key1] = daily_data[key2]

    def get_data(self):
        """Return collected data"""
        return self._data
